/*
 * do.c - do something
 * Feb 22 1994 created by Ikuo Nakagawa
 */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

/*
 * random and srandom are BSD function
 */
#if defined(H3050R) || defined(hpux)
#define srandom srand48
#define random lrand48
#elsif defined(SYSV)
#define srandom srand
#define random rand
#endif

/*
 * our include files
 */
#include "global.h"

/*
 * used in this file
 */
void do_reset();
void do_start();
void do_state();

/*
 *
 */
#define F_LEFT 1
#define F_RIGHT 2
#define F_BOTH (F_LEFT|F_RIGHT)
#define S_NONE 1
#define S_ACTIVE 2
#define S_HIT 3
#define COUNT 100

#define INTERVAL 100
#define MIN_LEVEL 1 /* low level */
#define MAX_LEVEL 8 /* high level */
#define MIN_SPEED 20 /* low level */
#define MAX_SPEED 3 /* high level */
#define SPEED(level) \
	(((MAX_SPEED - MIN_SPEED) * (level) \
		+ (MIN_SPEED * MAX_LEVEL - MAX_SPEED * MIN_LEVEL)) \
		 / (MAX_LEVEL - MIN_LEVEL))

/*
 * charactor structure
 */
struct chara_t {
	char hands; /* F_LEFT or F_RIGHT */
	char stat; /* S_NONE or S_ACTIVE or S_HIT */
	char code; /* ASCII charactor code */
	char tick;
	char rx;
	char ry;
	XtIntervalId iid; /* reary? */
};
static struct chara_t chars[] = {
	/* top line */
	{ F_LEFT,  S_ACTIVE, 'Q', -1, 0, 0, 0 },
	{ F_LEFT,  S_ACTIVE, 'W', -1, 1, 0, 0 },
	{ F_LEFT,  S_ACTIVE, 'E', -1, 2, 0, 0 },
	{ F_LEFT,  S_ACTIVE, 'R', -1, 3, 0, 0 },
	{ F_LEFT,  S_ACTIVE, 'T', -1, 4, 0, 0 },
	{ F_RIGHT, S_ACTIVE, 'Y', -1, 5, 0, 0 },
	{ F_RIGHT, S_ACTIVE, 'U', -1, 6, 0, 0 },
	{ F_RIGHT, S_ACTIVE, 'I', -1, 7, 0, 0 },
	{ F_RIGHT, S_ACTIVE, 'O', -1, 8, 0, 0 },
	{ F_RIGHT, S_ACTIVE, 'P', -1, 9, 0, 0 },
	/* middle line */
	{ F_LEFT,  S_ACTIVE, 'A', -1, 0, 1, 0 },
	{ F_LEFT,  S_ACTIVE, 'S', -1, 1, 1, 0 },
	{ F_LEFT,  S_ACTIVE, 'D', -1, 2, 1, 0 },
	{ F_LEFT,  S_ACTIVE, 'F', -1, 3, 1, 0 },
	{ F_LEFT,  S_ACTIVE, 'G', -1, 4, 1, 0 },
	{ F_RIGHT, S_ACTIVE, 'H', -1, 5, 1, 0 },
	{ F_RIGHT, S_ACTIVE, 'J', -1, 6, 1, 0 },
	{ F_RIGHT, S_ACTIVE, 'K', -1, 7, 1, 0 },
	{ F_RIGHT, S_ACTIVE, 'L', -1, 8, 1, 0 },
	{ F_RIGHT, S_ACTIVE, ';', -1, 9, 1, 0 },
	/* bottom line */
	{ F_LEFT,  S_ACTIVE, 'Z', -1, 0, 2, 0 },
	{ F_LEFT,  S_ACTIVE, 'X', -1, 1, 2, 0 },
	{ F_LEFT,  S_ACTIVE, 'C', -1, 2, 2, 0 },
	{ F_LEFT,  S_ACTIVE, 'V', -1, 3, 2, 0 },
	{ F_LEFT,  S_ACTIVE, 'B', -1, 4, 2, 0 },
	{ F_RIGHT, S_ACTIVE, 'N', -1, 5, 2, 0 },
	{ F_RIGHT, S_ACTIVE, 'M', -1, 6, 2, 0 },
	{ F_RIGHT, S_ACTIVE, ',', -1, 7, 2, 0 },
	{ F_RIGHT, S_ACTIVE, '.', -1, 8, 2, 0 },
	{ F_RIGHT, S_ACTIVE, '/', -1, 9, 2, 0 },
};

/*
 * for translation table
 */
static char trans[256];

/*
 * status structure
 */
struct state_t {
	int hands; /* specifies hand */
	int catch; /* catch and kill charactor */
	int miss; /* type miss */
	int over; /* timeover */
	int rest; /* rest to clear this stage */
	int sleep;
	int interval; /* current interval in [msec] */
	int sum_interval; /* sum of interval in [msec] */
	int level; /* level of player */
	int speed; /* speed of appearence */
	Pixel hit_pixel;
};
static struct state_t v = {
	F_BOTH,
	0, /* catch */
	0, /* miss */
	0, /* over */
	COUNT, /* rest */
	1000, /* sleep */
	2000, /* interval */
	0, /* sum_interval */
	3, /* level */
	SPEED(3), /* speed */
	0, /* hit_pixel */
};

/*
 * for original resources
 */
static XtResource resource[] = {
	{ "hitColor", "HitColor", XtRPixel, sizeof(Pixel),
		XtOffset(struct state_t *, hit_pixel), XtRString, "yellow" },
};

/*
 * static variables
 */
static Display *dpy;
static int scr;
static Window win;
static GC gc;
static XFontStruct *xfs;
static Pixmap frame;
static Pixel fg, bg, white, black;
static XtIntervalId tid = 0;

/*
 * initialize
 */
static void do_init(widget)
	Widget widget;
{
	XPoint tri[3];
	int i, depth;

	for (i = 0; i < XtNumber(trans); i++) {
		trans[i] = -1;
	}
	for (i = 0; i < XtNumber(chars); i++) {
		trans[(int)chars[i].code] = i;
		if (isupper(chars[i].code)) {
			trans[tolower(chars[i].code)] = i;
		}
	}

	dpy = XtDisplay(widget);
	scr = DefaultScreen(dpy);
	win = XtWindow(widget);
	white = WhitePixel(dpy, scr);
	black = BlackPixel(dpy, scr);
	depth = DefaultDepth(dpy, scr);

	XtVaGetApplicationResources(
		widget, (caddr_t)&v, resource, XtNumber(resource), NULL
	);

	XtVaGetValues(widget,
		XtNforeground, &fg,
		XtNbackground, &bg,
		XtNfont, &xfs,
		NULL
	);

	/*
	 * setup graphic context
	 */
	gc = XCreateGC(dpy, win, 0, 0);
	XSetForeground(dpy, gc, fg);
	XSetBackground(dpy, gc, bg);
	XSetFont(dpy, gc, xfs->fid);

	/*
	 * create frame pixmap
	 */
	frame = XCreatePixmap(dpy, win, LENGTH, LENGTH, depth);
	XSetForeground(dpy, gc, black);
	XFillRectangle(dpy, frame, gc, 0, 0, LENGTH, LENGTH);
	tri[0].x = 0;
	tri[0].y = 0;
	tri[1].x = 0;
	tri[1].y = LENGTH - 1;
	tri[2].x = LENGTH - 1;
	tri[2].y = 1 - LENGTH;
	XSetForeground(dpy, gc, white);
	XFillPolygon(
		dpy, frame, gc,
		tri, 3, Convex, CoordModePrevious
	);
	XSetForeground(dpy, gc, bg);
	XFillRectangle(
		dpy, frame, gc, PITCH, PITCH,
		LENGTH - PITCH * 2, LENGTH - PITCH * 2
	);

	/*
	 * re-setup gc
	 */
	XSetForeground(dpy, gc, fg);

	/*
	 * initialize for random()
	 */
	srandom(time(0));
}

/*
 * simple one
 */
void do_exit()
{
	exit(0);
}

/*
 * XtCallbackProc
 */
void do_hands(widget, mask, data)
	Widget widget;
	int mask;
	XtPointer data; /* ignore */
{
	v.hands = mask;
	do_reset();
	do_state();
}

/*
 * XtCallbackProc
 */
void do_level(widget, level, data)
	Widget widget;
	int level;
	XtPointer data; /* ignore */
{
	v.level = level;
	do_reset();
	do_state();
}

/*
 * echo a charactor - give ID of charactor as a paramter
 */
static void do_echo(i)
	int i;
{
	struct chara_t *cp;
	int x, y;

	cp = &chars[i];
	x = cp->rx * (PITCH + LENGTH) + cp->ry * PITCH + PITCH * 2;
	y = cp->ry * (PITCH + LENGTH) + PITCH * 2;
	if (cp->stat == S_NONE) {
		XClearArea(dpy, win, x, y, LENGTH, LENGTH, False);
	} else {
		XCopyArea(dpy, frame, win, gc, 0, 0, LENGTH, LENGTH, x, y);
		x += LENGTH / 2 - XTextWidth(xfs, &cp->code, 1) / 2;
		y += LENGTH / 2 + xfs->ascent / 2;
		XSetForeground(dpy, gc,
			(cp->stat == S_ACTIVE ? fg : v.hit_pixel));
		XDrawString(dpy, win, gc, x, y, &cp->code, 1);
	}
}

/*
 *
 */
static void do_error()
{
	XBell(dpy, -20); XFlush(dpy);
	v.miss++;
	do_state();
}

/*
 * catch a charactor from keyboard
 */
static void do_catch(i)
	int i;
{
	if (chars[i].stat != S_ACTIVE) {
		do_error();
		return;
	}
	XtRemoveTimeOut(chars[i].iid);
	chars[i].iid = 0;
	chars[i].stat = S_HIT;
	do_echo(i);
	v.catch++;
	do_state();
	chars[i].stat = S_NONE;
	XtAppAddTimeOut(app, 300, do_echo, (XtPointer)i);
}

/*
 *
 */
void do_timeover(i, id)
	int i;
	XtIntervalId *id;
{
	chars[i].iid = 0; /* clear interval id */
	chars[i].stat = S_NONE;
	do_echo(i);
	v.over++;
	do_state();
}

/*
 *
 */
static void do_tick(data, iid)
	XtPointer data; /* closure */
	XtIntervalId *iid; /* id */
{
	int i;

	if (0 < v.rest && random() % v.speed == 0) {
		i = random() % XtNumber(chars);
		if (chars[i].iid == 0 && (chars[i].hands & v.hands)) {
			chars[i].stat = S_ACTIVE;
			chars[i].iid = XtAppAddTimeOut(
				app, v.interval, do_timeover, (XtPointer)i
			);
			do_echo(i);
			v.rest--;
			do_state();
		}
	}

	if (v.over + v.catch < COUNT) {
		tid = XtAppAddTimeOut(app, INTERVAL, do_tick, 0);
	} else {
		do_start(); /* toggle, i.e stop */
	}
}

/*
 *
 */
void do_reset()
{
	/* reset all state */
	v.catch = 0;
	v.miss = 0;
	v.over = 0;
	v.rest = COUNT;
	v.sleep = 1000; /* 1 sec */
	v.interval = 1500 + 200 * (MAX_LEVEL - v.level);
	v.speed = SPEED(v.level);
	if (v.hands != F_BOTH) v.speed /= 2;
}

/*
 *
 */
void do_start(widget, adjust)
	Widget widget;
	Widget adjust;
{
	static Widget start = NULL, setup;
	int i;

	/* first save widget variable - only once */
	if (start == NULL) start = widget, setup = adjust;

	if (tid) { /* if already start - stop it */

		for (i = 0; i < XtNumber(chars); i++) {
			if (chars[i].stat == S_ACTIVE && chars[i].iid) {
				XtRemoveTimeOut(chars[i].iid);
				chars[i].iid = 0;
			} else {
				chars[i].stat = S_ACTIVE;
			}
			do_echo(i);
		}
		XtVaSetValues(start, XtNlabel, "Start", NULL);
		XtVaSetValues(setup, XtNsensitive, True, NULL);
		XtRemoveTimeOut(tid); tid = 0;

	} else { /* start! reset all code */

		do_reset();
		for (i = 0; i < XtNumber(chars); i++) {
			chars[i].stat = S_NONE;
			do_echo(i);
		}
		XtVaSetValues(start, XtNlabel, "Stop", NULL);
		XtVaSetValues(setup, XtNsensitive, False, NULL);
		tid = XtAppAddTimeOut(app, 1000, do_tick, 0);

	}

	/* set score */
	do_state();

}

/*
 * redraw all keys
 */
void do_expose(widget, data, event, flags)
	Widget widget;
	XtPointer data;
	XExposeEvent *event;
	Boolean *flags;
{
	int i;

	if (0 == dpy) do_init(widget);

	XClearWindow(dpy, win);

	for (i = 0; i < XtNumber(chars); i++) {
		do_echo(i);
	}
}

/*
 * an action for key event
 */
void act_keyin(widget, event, param, nparam)
	Widget widget;
	XKeyEvent *event;
	String *param;
	Cardinal *nparam;
{
	unsigned char ch;

	if (!tid) return;

	/*
	 * lookup input key
	 */
	if (!XLookupString(event, &ch, 1, 0, 0)) return;

	/*
	 * echo one if tick is ture
	 */
	if (trans[ch] < 0) do_error(); else do_catch(trans[ch]);
}

/*
 * simple macro
 */
#define STRLAST(s) ((char *)(s) + strlen(s))

/*
 * set labels
 */
void do_state(widget)
	Widget widget;
{
	static Widget label = NULL;
	char buffer[1024];
	int hands;

	/* save widget variable - only once */
	if (label == NULL) label = widget;

	hands = (v.hands == F_LEFT ? 'L' : (v.hands == F_RIGHT ? 'R' : 'B'));
	sprintf(buffer, "<LEVEL %d(%c)>", v.level, hands);
	sprintf(STRLAST(buffer), " Rest: %3d", v.rest);
	sprintf(STRLAST(buffer), ", Hit: %3d", v.catch);
	sprintf(STRLAST(buffer), ", Lost: %3d", v.over);
	sprintf(STRLAST(buffer), ", Error: %3d", v.miss);
	XtVaSetValues(label, XtNlabel, buffer, NULL);
}
