/*
 * main.c - 1994/02/20 created by Ikuo Nakagawa
 *   This file contains main() functions for *xtypo*
 */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>

/*
 * our include files
 */
#include "global.h"

/*
 * external functions
 */
void do_exit();
void do_level();
void do_start();
void do_expose();
void do_hands();
void do_state();
void act_keyin();

/*
 * global variables for Xt
 */
XtAppContext app;
Widget top;

/*
 * fallback resouces
 */
static char *fallback[] = {

	/* for global defs */
	"*Foreground : dark green",
	"*Background : gray",
	"*BorderColor : black",
	"*BorderWidth : 1",
	"*Font : -adobe-helvetica-bold-o-normal--14-*-iso8859-*",
	"*Title	: xtypo",
	"*IconName : xtypo",
	"*ShapeStyle : ShapeOval",
	"*Resize : False",
	"*Command.width : 80",
	"*Command.height : 20",
	"*MenuButton.width : 80",
	"*MenuButton.height : 20",

	/* for each widget */
	"*exit.label : Exit",
	"*start.label : Start",
	"*setup.label : Setup",
	"*state.justify : Left",
	"*state.font : -sony-*-*-r-*--16-*-100-100-c-*-iso8859-*",
	"*board.font : -adobe-helvetica-medium-r-normal--34-*",

	/* terminater */
	NULL,
};

/*
 * main procedure
 */
void main(argc, argv)
int argc;
char **argv;
{
	Widget base, button, setup, start, w, v;
	int i;
	static char trans_base[] =
		"<Key>Escape : act_exit() \n\
		 <KeyPress> : act_keyin()";
	static XtActionsRec action[] = {
		{ "act_keyin", act_keyin },
		{ "act_exit", do_exit },
	};

	/*
	 * Initialize X Window System
	 */
	top = XtVaAppInitialize(
		&app, CLASS, 0, 0, &argc, argv, fallback, NULL
	);

	/*
	 * setup action table
	 */
	XtAppAddActions(app, action, XtNumber(action));

	/*
	 * base
	 */
	base = XtVaCreateManagedWidget(
		"base", formWidgetClass, top, NULL
	);

	/*
	 * set translation table
	 */
	XtOverrideTranslations(base, XtParseTranslationTable(trans_base));

	/*
	 * for EXIT button
	 */
	button = w = XtVaCreateManagedWidget(
		"exit", commandWidgetClass, base, NULL
	);
	XtAddCallback(w, XtNcallback, do_exit, 0);

	/*
	 * for SETUP label
	 */
	setup = w = XtVaCreateManagedWidget(
		"setup", menuButtonWidgetClass, base,
		XtNmenuName, "setup-menu",
		XtNfromHoriz, button, NULL
	);
	w = XtVaCreatePopupShell(
		"setup-menu", simpleMenuWidgetClass, w, NULL
	);
	for (i = 1; i <= 8; i++) {
		Widget v;
		char name[10];
		char label[10];
		sprintf(name, "level-%d", i);
		sprintf(label, " Level %d ", i);
		v = XtVaCreateManagedWidget(
			name, smeBSBObjectClass, w, XtNlabel, label, NULL
		);
		XtAddCallback(v, XtNcallback, do_level, (XtPointer)i);
	}
	XtVaCreateManagedWidget(
		"smeLine", smeLineObjectClass, w, NULL
	);
	v = XtVaCreateManagedWidget(
		"left", smeBSBObjectClass, w,
		XtNlabel, " Left Hand ", NULL
	);
	XtAddCallback(v, XtNcallback, do_hands, (XtPointer)1);
	v = XtVaCreateManagedWidget(
		"right", smeBSBObjectClass, w,
		XtNlabel, " Right Hand ", NULL
	);
	XtAddCallback(v, XtNcallback, do_hands, (XtPointer)2);
	v = XtVaCreateManagedWidget(
		"both", smeBSBObjectClass, w,
		XtNlabel, " Both Hands ", NULL
	);
	XtAddCallback(v, XtNcallback, do_hands, (XtPointer)3);

	/*
	 * for START button
	 */
	start = w = XtVaCreateManagedWidget(
		"start", commandWidgetClass, base,
		XtNfromHoriz, setup, NULL
	);
	XtAddCallback(w, XtNcallback, do_start, setup);

	/*
	 * for Status label
	 */
	w = XtVaCreateManagedWidget(
		"state", labelWidgetClass, base,
		XtNwidth, pixmap_width,
		XtNheight, 20,
		XtNborderWidth, 0,
		XtNfromVert, button,
		XtNfromHoriz, NULL, NULL
	);
	do_state(w);

	/*
	 * for main board
	 */
	w = XtVaCreateManagedWidget(
		"board", labelWidgetClass, base,
		XtNlabel, "",
		XtNwidth, pixmap_width,
		XtNheight, pixmap_height,
		XtNfromVert, w, NULL
	);
	XtAddEventHandler(w, ExposureMask, False, do_expose, 0);

	/*
	 * Realize All Widget
	 */
	XtRealizeWidget(top);

	/*
	 * now, start main loop - will not return
	 */
	XtAppMainLoop(app);
}
